ElectricVehicle = {
	MOD_DIR = g_currentModDirectory,
	prerequisitesPresent = function (specializations)
		return SpecializationUtil.hasSpecialization(Enterable, specializations) and SpecializationUtil.hasSpecialization(Motorized, specializations)
	end,
	registerFunctions = function (vehicleType)
	end
}

function ElectricVehicle.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getCanMotorRun", ElectricVehicle.getCanMotorRun)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "getMotorNotAllowedWarning", ElectricVehicle.getMotorNotAllowedWarning)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "updateConsumers", ElectricVehicle.updateConsumers)
end

function ElectricVehicle.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", ElectricVehicle)
	SpecializationUtil.registerEventListener(vehicleType, "onEnterVehicle", ElectricVehicle)
	SpecializationUtil.registerEventListener(vehicleType, "onLeaveVehicle", ElectricVehicle)
end

function ElectricVehicle:onLoad(savegame)
	local spec = self.spec_motorized
	spec.batteryEmptyWarning = g_i18n:getText("warning_motorBatteryEmpty", self.customEnvironment)
end

function ElectricVehicle:onEnterVehicle(isControlling)
	if isControlling and g_currentMission ~= nil and g_currentMission.hud ~= nil then
		local speedMeter = g_currentMission.hud.speedMeter
		local iconElement = speedMeter.fuelGaugeIconElement

		iconElement:setImage(self.baseDirectory .. "hud_battery_icon.png")
		iconElement:setColor()

		iconElement.normalUVs = iconElement.overlay.uvs
		iconElement.normalOffset = iconElement.overlay.offsetY

		iconElement:setUVs(Overlay.DEFAULT_UVS)
		iconElement:setScale(0.85, 0.85)

		local _, offsetY = getNormalizedScreenValues(0, 2)
		iconElement.overlay.offsetY = offsetY

		for _, element in pairs(speedMeter.fuelGaugeSegmentPartElements) do
			element:setColor(SpeedMeterDisplay.COLOR.DAMAGE_GAUGE[1], SpeedMeterDisplay.COLOR.DAMAGE_GAUGE[2], SpeedMeterDisplay.COLOR.DAMAGE_GAUGE[3])
		end

		speedMeter.electricVehicle = self
	end
end

function ElectricVehicle:onLeaveVehicle()
	if g_currentMission ~= nil and g_currentMission.hud ~= nil then
		local speedMeter = g_currentMission.hud.speedMeter

		if speedMeter.electricVehicle == self then
			local iconElement = speedMeter.fuelGaugeIconElement

			iconElement:setImage(g_baseHUDFilename)
			iconElement:setColor()
			iconElement:setUVs(iconElement.normalUVs)
			iconElement:setScale(1, 1)

			iconElement.overlay.offsetY = iconElement.normalOffset

			for _, element in pairs(speedMeter.fuelGaugeSegmentPartElements) do
				element:setColor(SpeedMeterDisplay.COLOR.FUEL_GAUGE[1], SpeedMeterDisplay.COLOR.FUEL_GAUGE[2], SpeedMeterDisplay.COLOR.FUEL_GAUGE[3])
			end

			speedMeter.electricVehicle = false
		end
	end
end

function ElectricVehicle:getCanMotorRun(superFunc)
	local spec = self.spec_motorized
	local canRun = true

	if spec.consumersByFillTypeName.electricCharge ~= nil then
		canRun = self:getFillUnitFillLevel(spec.consumersByFillTypeName.electricCharge.fillUnitIndex) > 0
	end

	return canRun and superFunc(self)
end

function ElectricVehicle:getMotorNotAllowedWarning(superFunc)
	local spec = self.spec_motorized

	if spec.consumersByFillTypeName.electricCharge ~= nil and self:getFillUnitFillLevel(spec.consumersByFillTypeName.electricCharge.fillUnitIndex) <= 0 then
		return spec.batteryEmptyWarning
	end

	return superFunc(self)
end

function ElectricVehicle:updateConsumers(superFunc, dt, accInput)
	local spec = self.spec_motorized
	local aiBuysCharge = g_currentMission.missionInfo.helperBuyFuel and self:getIsAIActive()
	local startFillLevel = -1
	local consumer = spec.consumersByFillTypeName.electricCharge

	if aiBuysCharge and consumer ~= nil then
		startFillLevel = self:getFillUnitFillLevel(consumer.fillUnitIndex)
	end

	superFunc(self, dt, accInput)

	if aiBuysCharge and consumer ~= nil then
		local endFillLevel = self:getFillUnitFillLevel(consumer.fillUnitIndex)
		local used = startFillLevel - endFillLevel

		if used > 0 then
			local fillType = self:getFillUnitLastValidFillType(consumer.fillUnitIndex)
			local price = used * g_currentMission.economyManager:getPricePerLiter(fillType) * 1.5
			local stats = g_currentMission:farmStats(self:getOwnerFarmId())

			stats:updateStats("expenses", price)
			g_currentMission:addMoney(-price, self:getOwnerFarmId(), MoneyType.PURCHASE_FUEL, true)
			self:addFillUnitFillLevel(self:getOwnerFarmId(), consumer.fillUnitIndex, used, fillType, ToolType.UNDEFINED)
		end
	end
end

function SpeedMeterDisplay.getVehicleFuelLevelAndCapacity(vehicle)
	local fillUnitIndex = vehicle:getConsumerFillUnitIndex(FillType.DIESEL)

	if fillUnitIndex == nil then
		fillUnitIndex = vehicle:getConsumerFillUnitIndex(FillType.ELECTRICCHARGE)
	end

	local level = vehicle:getFillUnitFillLevel(fillUnitIndex)
	local capacity = vehicle:getFillUnitCapacity(fillUnitIndex)

	return level, capacity
end

local oldMakeDisplayItem = ShopController.makeDisplayItem

function ShopController:makeDisplayItem(storeItem, concreteItem)
	local displayItem = oldMakeDisplayItem(self, storeItem, concreteItem)

	if storeItem.specs ~= nil and storeItem.specs.fuel ~= nil then
		local fillUnits = storeItem.specs.fuel.fillUnits

		if fillUnits ~= nil then
			fillUnits = fillUnits[1]

			if fillUnits ~= nil then
				for i = 1, #fillUnits do
					local fillUnit = fillUnits[i]

					if fillUnit.fillTypes ~= nil and fillUnit.fillTypes:lower():find("electriccharge") ~= nil and fillUnit.capacity > 0 then
						for j = 1, #displayItem.attributeIconProfiles do
							if displayItem.attributeIconProfiles[j] == "shopListAttributeIconPower" then
								table.insert(displayItem.attributeIconProfiles, j + 1, "shopListAttributeIconElectricCapacity")
								table.insert(displayItem.attributeValues, j + 1, string.format(g_i18n:getText("shop_electricCapacity"), fillUnit.capacity))

								break
							end
						end
					end
				end
			end
		end
	end

	return displayItem
end

local oldProcessAttributeData = ShopConfigScreen.processAttributeData

function ShopConfigScreen:processAttributeData(storeItem, concreteItem)
	oldProcessAttributeData(self, storeItem, concreteItem)

	if FillType.ELECTRICCHARGE ~= nil then
		local storeItems = {}

		if storeItem.bundleInfo == nil then
			table.insert(storeItems, storeItem)
		else
			for _, itm in ipairs(storeItem.bundleInfo.bundleItems) do
				table.insert(storeItems, itm.item)
			end
		end

		local electricCapacity = 0

		for _, item in ipairs(storeItems) do
			electricCapacity = electricCapacity + self:processStoreItemFuelCapacity(item, FillType.ELECTRICCHARGE)
		end

		if electricCapacity > 0 then
			local insertIndex = -1

			for i = #self.attrIcon - 1, 1, -1 do
				if self.attrIcon[i].visible then
					if self.attrIcon[i].profile == "shopListAttributeIconPower" then
						self.attrIcon[i + 1]:applyProfile("shopListAttributeIconElectricCapacity")
						self.attrIcon[i + 1]:setVisible(true)

						insertIndex = i + 1

						break
					end

					self.attrIcon[i + 1]:applyProfile(self.attrIcon[i].profile)
					self.attrIcon[i + 1]:setVisible(true)
				end
			end

			if insertIndex > -1 then
				for i = #self.attrValue - 1, 1, -1 do
					if self.attrValue[i].text ~= "" then
						if i < insertIndex then
							self.attrValue[i + 1]:setText(string.format(g_i18n:getText("shop_electricCapacity"), electricCapacity))

							break
						end

						self.attrValue[i + 1]:setText(self.attrValue[i].text)
					end
				end
			end
		end
	end
end

if g_gui ~= nil then
	for name, profile in pairs(g_gui.profiles) do
		if name == "shopListAttributeIconPower" then
			local newProfile = GuiProfile:new(g_gui.profiles, g_gui.traits)
			newProfile.parent = profile.parent
			newProfile.values = ListUtil.copyTableRecursively(profile.values)
			newProfile.values.imageFilename = ElectricVehicle.MOD_DIR .. "hud_battery_icon.png"
			newProfile.values.imageSize = "96 96"
			newProfile.values.imageUVs = "0px 0px 96px 96px"
			g_gui.profiles.shopListAttributeIconElectricCapacity = newProfile
		end
	end
end
